# coding: utf-8

'''Add V-Ray OpenSubdiv Attribute Group to mesh nodes

How to use: Regist to shelf following 2 line as Python

import HtVRayAddOpenSubdivAttrGrp
HtVRayAddOpenSubdivAttrGrp.main()

Auther: Hiroshi Takashima, Canplaize Inc.
License: CC BY-SA 4.0
'''

import pymel.core as pm
from pymel.core.language import Mel as mel

try:
    from PySide2.QtWidgets import *
    from PySide2.QtCore import *
    from PySide2.QtGui import *
    pysideEnv = 2
except ImportError:
    from PySide.QtCore import *
    from PySide.QtGui import *
    pysideEnv = 1

from maya.app.general.mayaMixin import MayaQWidgetBaseMixin

if pysideEnv == 1:
	import HtVRayAddOpenSubdivAttrGrpUI as HtVRayAddOpenSubdivAttrGrpUI
	#reload(HtVRayAddOpenSubdivAttrGrpUI)
elif pysideEnv == 2:
	import HtVRayAddOpenSubdivAttrGrpUI2 as HtVRayAddOpenSubdivAttrGrpUI
	#reload(HtVRayAddOpenSubdivAttrGrpUI2)

VERSION = 1.0

class HtAddOpenSubdivAttrGrpDialog(MayaQWidgetBaseMixin, QWidget):
	def __init__(self,*args,**kwargs):
		super(HtAddOpenSubdivAttrGrpDialog, self).__init__(*args,**kwargs)
		#self.closeOtherInstance()
		self.setAttribute(Qt.WA_DeleteOnClose, True)
		self.setObjectName(self.__class__.__name__)
		self.setProperty("saveWindowPref", True)
		self.initUI()
		#print self.parent().objectName()

	def initUI(self):
		self.ui = HtVRayAddOpenSubdivAttrGrpUI.Ui_AddOpenSubdivAttrGrpDialog()
		self.ui.setupUi(self)
		self.setWindowTitle(self.windowTitle() + " ver " + VERSION.__str__());
		#add OsdType Menu
		self.ui.CB_OsdType.addItems(['Catmull-Clark','Loop'])
		self.ui.CB_OsdType.setCurrentIndex(0)
		self.ui.CB_OsdPreserveMapBorders.addItems(['None','Internal','All'])
		self.ui.CB_OsdPreserveMapBorders.setCurrentIndex(1)
		#addtional slot
		self.ui.buttonBox.button(QDialogButtonBox.Apply).clicked.connect(self.clickApply)
		#change enable/disable checkbox management
		self.ui.CB_Activate.manageCheckBox = [
			self.ui.Change_OsdRenderAsOpenSubdivSurface,
			self.ui.Change_OsdDepth,
			self.ui.Change_OsdType,
			self.ui.Change_OsdPreserveMapBorders,
			self.ui.Change_OsdSubdivideUVs,
			self.ui.Change_OSDPreserveGeometryBorders
		]
		self.ui.Change_OsdRenderAsOpenSubdivSurface.manageWidget = [self.ui.CB_OsdRenderAsOpenSubdivSurface]
		self.ui.Change_OsdDepth.manageWidget = [self.ui.label_OsdDepth, self.ui.LE_OsdDepth, self.ui.slider_OsdDepth]
		self.ui.Change_OsdType.manageWidget = [self.ui.label_OsdType, self.ui.CB_OsdType]
		self.ui.Change_OsdPreserveMapBorders.manageWidget = [self.ui.label_OsdPreserveMapBorders,self.ui.CB_OsdPreserveMapBorders]
		self.ui.Change_OsdSubdivideUVs.manageWidget = [self.ui.CB_OsdSubdivideUVs]
		self.ui.Change_OSDPreserveGeometryBorders.manageWidget = [self.ui.CB_OSDPreserveGeometryBorders]

		for checkbox in self.ui.CB_Activate.manageCheckBox:
			for widget in checkbox.manageWidget:
				widget.setEnabled(False)
	#slot
	def closeOtherInstance(self):
		'''Close other instance
		'''
		print self.parent()
		[widget.close() for widget in self.parent().findChildren(self.__class__) if widget is not self]

	def activeCheckChanged(self,state):
		self.setEnableAllWidget(self.sender(), state)

	def updateDepthSlider(self):
		'''Update Line Edit, When Slider value is changed
		'''
		self.ui.LE_OsdDepth.setText(str(self.ui.slider_OsdDepth.value()))

	def updateDepth(self):
		'''Update Slider, Whem Line Edit string is changed
		'''
		self.ui.slider_OsdDepth.setValue(int(self.ui.LE_OsdDepth.text()))

	def clickClose(self):
		self.close()

	def clickApply(self):
		self.filterMeshShapeAndExec()

	def changeCheckState(self,state):
		self.setEnableManagedWidget(self.sender(), state)

	#end of slot

	def setEnableManagedWidget(self,checkbox,state):
		if hasattr(checkbox, "manageWidget"):
			for w in checkbox.manageWidget:
				w.setEnabled(state)

	def setEnableAllWidget(self,rootCheckbox,state):
		if hasattr(rootCheckbox, "manageCheckBox"):
			for checkbox in rootCheckbox.manageCheckBox:
				checkbox.setEnabled(state)
				self.setEnableManagedWidget(checkbox, checkbox.isChecked() and state)

	def filterMeshShapeAndExec(self):
		'''filter mesh from selected nodes exclude intermediate objects
		'''
		targetMeshShapes = []
		for node in pm.selected():
			#If selected is transform, get mesh shapes which is not intermediate object.
			if node.nodeType() == "transform":
				targetMeshShapes.extend([shape for shape in node.getShapes() if (shape.nodeType() == "mesh") and (not shape.intermediateObject.get())])
			#Else if selected is mash shape, only append to target list
			elif node.nodeType() == "mesh":
				targetMeshShapes.append(node)

		for meshNode in targetMeshShapes:
			self.addOpenSubdivAttrGrpToSingleMesh(meshNode)
					
	def addOpenSubdivAttrGrpToSingleMesh(self,shape):
		'''Add V-Ray OpenSubdiv Attribute group to mesh shape node, and turn on preview smooth mesh 
		'''
		state = self.ui.CB_Activate.isChecked()
		hasVrayOpenSubdivAttrs = pm.attributeQuery("vraySeparator_vray_opensubdiv",node=shape,exists=True)
		if (not hasVrayOpenSubdivAttrs) and state:
			#if attrGrp not exists and activate is True, add attrGrp
			mel.eval('vray addAttributesFromGroup "{meshname}" "vray_opensubdiv" 1;'.format(meshname=shape.name()))
		elif hasVrayOpenSubdivAttrs and (not state):
			#if attrGrp already exists and activate is False, remove attrGroup
			mel.eval('vray addAttributesFromGroup "{meshname}" "vray_opensubdiv" 0;'.format(meshname=shape.name()))

		if state:
			pm.displaySmoothness(divisionsU=3, divisionsV=3, pointsWire=16, pointsShaded=4, polygonObject=3) #active preview smooth mesh
			shape.osdVertBoundary.set(2)
			if self.ui.Change_OsdRenderAsOpenSubdivSurface.isChecked():
				shape.vrayOsdSubdivEnable.set(1 if self.ui.CB_OsdRenderAsOpenSubdivSurface.isChecked() else 0)
			if self.ui.Change_OsdDepth.isChecked():
				shape.vrayOsdSubdivDepth.set(int(self.ui.LE_OsdDepth.text()))
			if self.ui.Change_OsdType.isChecked():
				shape.vrayOsdSubdivType.set(self.ui.CB_OsdType.currentIndex())
			if self.ui.Change_OsdPreserveMapBorders.isChecked():
				shape.vrayOsdPreserveMapBorders.set(self.ui.CB_OsdPreserveMapBorders.currentIndex())
			if self.ui.Change_OsdSubdivideUVs.isChecked():
				shape.vrayOsdSubdivUVs.set(1 if self.ui.CB_OsdSubdivideUVs.isChecked() else 0)
			if self.ui.Change_OSDPreserveGeometryBorders.isChecked():
				shape.vrayOsdPreserveGeomBorders.set(1 if self.ui.CB_OSDPreserveGeometryBorders.isChecked() else 0)
		else:
			pm.displaySmoothness(polygonObject=0) #deactive preview smooth mesh
			shape.osdVertBoundary.set(1)

def main():
	dlg = HtAddOpenSubdivAttrGrpDialog()
	dlg.show()
	return dlg

if __name__ == '__main__':
	main()